from django.db.models import Q
from django.test import TestCase
from django_auth_ldap.backend import LDAPBackend, populate_user
from django.contrib.auth.models import User, Group

class LDAPGroupMapTestCase(TestCase):
    fixtures = ['users.json',
                'groups.json',
                'ldap_groups.json']

    def setUp(self):
        self.admin_user_group_dns = set([
            u'cn=app admins,ou=groups,dc=example,dc=local',
            u'cn=app users,ou=groups,dc=example,dc=local'])
        self.non_admin_user_group_dns = set([
            u'cn=app users,ou=groups,dc=example,dc=local'])
        self.admin_user = User.objects.get(username = 'admin')
        self.nonadmin_user = User.objects.get(username = 'user')
        # Our users have not been assigned to any django groups yet, if this
        # raises an exception, there is something wrong with our fixtures
        self.assertEquals(self.admin_user.groups.count(), 0)
        self.assertEquals(self.nonadmin_user.groups.count(), 0)


    def test_admin_user_group_mapping(self):
        # Test the group map for the 'admin' user
        ldap_user = User()
        ldap_user.group_dns = self.admin_user_group_dns
        populate_user.send(
            LDAPBackend, user = self.admin_user, ldap_user = ldap_user)

        # According to our group mapping, the 'admin' user should be populated
        # to both the 'Admins' and the 'Users' Django groups.
        self.assertTrue(self.admin_user.groups.filter(name = 'Admins'))
        self.assertTrue(self.admin_user.groups.filter(name = 'Users'))

        self.assertEquals(Group.objects.filter(
            user__username = 'admin').count(), 2)

        self.assertFalse(self.admin_user.groups.filter(
            name = 'nonexistentgroup'))

        # Ensure Django groups are updated when LDAP group membership changes.
        ldap_user.group_dns = set([
            u'cn=app users,ou=groups,dc=example,dc=local'])
        populate_user.send(
            LDAPBackend, user = self.admin_user, ldap_user = ldap_user)
        self.assertFalse(self.nonadmin_user.groups.filter(
            name = 'Admins'))
        self.assertEquals(Group.objects.filter(
            user__username = 'admin').count(), 1)

        
    def test_nonadmin_user_group_mapping(self):
        # Test the mapping for the non-admin user, 'user'.
        ldap_user = User()
        ldap_user.group_dns = self.non_admin_user_group_dns
        populate_user.send(
            LDAPBackend, user = self.nonadmin_user, ldap_user = ldap_user)

        # According to our group mapping, the 'user' user should be populated
        # to only the 'Users' Django group.
        self.assertFalse(self.nonadmin_user.groups.filter(
            name = 'Admins'))
        self.assertTrue(self.nonadmin_user.groups.filter(
            name = 'Users'))

        self.assertEquals(Group.objects.filter(
            user__username = 'user').count(), 1)

        self.assertFalse(self.nonadmin_user.groups.filter(
            name = 'nonexistentgroup'))

        # Ensure Django groups are updated when LDAP group membership changes.
        ldap_user.group_dns = set([])
        populate_user.send(
            LDAPBackend, user = self.nonadmin_user, ldap_user = ldap_user)
        self.assertFalse(self.nonadmin_user.groups.filter(
            name = 'Users'))
        self.assertEquals(Group.objects.filter(
            user__username = 'user').count(), 0)



import ldap, logging
from ldap import SERVER_DOWN
from django.db import models
from django.conf import settings
from django.dispatch import receiver
from django.contrib.auth.models import Group
from django_auth_ldap.backend import LDAPBackend, populate_user
logger = logging.getLogger(__name__)

server_uri = settings.AUTH_LDAP_SERVER_URI
bind_dn = settings.AUTH_LDAP_BIND_DN
bind_password = settings.AUTH_LDAP_BIND_PASSWORD
ldap_groups_search = settings.AUTH_LDAP_GROUP_SEARCH

def get_ldap_groups():
    dn_choices = []
    l = ldap.initialize(server_uri)
    try:
        l.simple_bind_s(bind_dn, bind_password)
    except SERVER_DOWN, e:
        logger.warning('%s: %s' % (e.message.get('desc'), server_uri))
        return dn_choices
    for dn, entry in ldap_groups_search.execute(l):
        dn_choices += [(dn, entry['cn'][0])]
    return dn_choices

class LDAPGroupMap(models.Model):
    ldap_group = models.CharField(
        unique = True,
        verbose_name = 'LDAP Group',
        choices = get_ldap_groups(),
        max_length = 255)

    django_group = models.ManyToManyField(Group)
    
    def __unicode__(self):
        django_groups = ', '.join(i.name for i in self.django_group.all())
        return '%s -> %s' % (self.ldap_group, django_groups)
    
    class Meta:
        ordering = ('ldap_group',)


@receiver(populate_user, sender = LDAPBackend)
def set_group_perms(sender, **kwargs):
    '''
    Make sure that the user's group membership is always up to date.
    Note: Updates only occur when a user authenticates.
    '''
    user = kwargs['user']
    ldap_user = kwargs['ldap_user']
    user.groups.clear()
    for group_map in LDAPGroupMap.objects.all():
        dn = group_map.ldap_group.lower()
        if dn in ldap_user.group_dns:
            user.groups.add(*group_map.django_group.filter(
                ldapgroupmap__ldap_group__iexact = dn))
            user.save()



import ldap
from django_auth_ldap.config import LDAPSearch, ActiveDirectoryGroupType
from common_config import users_ou, groups_ou

# See the following url for more information about AUTH_LDAP_GROUP_TYPE:
# https://pythonhosted.org/django-auth-ldap/reference.html?highlight=grouptype#django_auth_ldap.config.LDAPGroupType

AUTH_LDAP_GROUP_TYPE = ActiveDirectoryGroupType()

# In the example below, the user search will only match users with accounts that are _not_ disabled. Refer to the section
# titled "Further Reading" in this package's README.md for more information about search scopes.

AUTH_LDAP_USER_SEARCH = LDAPSearch(
    users_ou, ldap.SCOPE_ONELEVEL, '(&(sAMAccountName=%(user)s)(!(userAccountControl:1.2.840.113556.1.4.803:=2)))')

AUTH_LDAP_GROUP_SEARCH = LDAPSearch(
    groups_ou, ldap.SCOPE_ONELEVEL, '(objectClass=group)')



#########################################################
# This is an example ``settings.py`` DO NOT use this    #
# in production, it is for demonstration purposes only. #
#########################################################

## First, we begin with the minimum required settings that
## django will need in order to run the demonstration.

import sys

DEBUG = True
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'ldap_groups_test_db'
    }
}

INSTALLED_APPS = (
    'django.contrib.staticfiles',
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'ldap_groups',                  ## The django-ldap-group-mapper ...
    'myapp')                        ## ... and our "demo" app

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware'
)


ROOT_URLCONF = 'myapp.urls'

STATIC_URL='/static/'
STATIC_ROOT='staticfiles'

SECRET_KEY = 'XXXxxxXXXxxxXXXxxxXXXxxxXXXxxx'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'standard': {
            'format': '[%(asctime)s] [%(levelname)s] [%(name)s] %(message)s'
        }
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
            'formatter': 'standard'
        }
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True
        }
    }
}

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'django_auth_ldap.backend.LDAPBackend'
)

## Next, import the configuration options common to most (all?)
## LDAP implementations:

from Example_LDAP_Configs.common_config import *

## Finally, uncomment the line(s) that correspond to the LDAP
## implementation that you are running.

## OpenLDAP:

from Example_LDAP_Configs.openldap_config import *

## Microsoft Active Directory:

#from Example_LDAP_Configs.msad_config import *



ldap_domain = 'dc=example,dc=local'
groups_ou = 'ou=Groups,dc=example,dc=local'
users_ou = 'ou=Users,dc=example,dc=local'

AUTH_LDAP_SERVER_URI = 'ldap://localhost'
AUTH_LDAP_BIND_DN = 'uid=lbind,%s' % users_ou
AUTH_LDAP_BIND_PASSWORD = 'password'
AUTH_LDAP_REQUIRE_GROUP = 'cn=All Users,%s' % groups_ou
AUTH_LDAP_ALWAYS_UPDATE_USER = True
AUTH_LDAP_MIRROR_GROUPS = False
AUTH_LDAP_USER_FLAGS_BY_GROUP = {
    'is_staff': 'cn=App Admins,%s' % groups_ou,
    'is_active': 'cn=All Users,%s' % groups_ou,
    'is_superuser': 'cn=App Admins,%s' % groups_ou
}
AUTH_LDAP_USER_ATTR_MAP = {
    'first_name': 'givenName',
    'last_name': 'sn',
    'email': 'mail'
}



from setuptools import find_packages, setup

version = '0.1.2'
github_url = 'https://github.com/musashiXXX/django-ldap-group-mapper'
long_description = 'A simple LDAP group -> Django group mapper, for use with' \
                   + ' django-auth-ldap.'

setup(
  name = 'django-ldap-group-mapper',
  version = version,
  author = 'Charles Hamilton',
  author_email = 'musashi@nefaria.com',
  packages = find_packages(exclude = [
    '*.tests', '*.tests.*','tests.*','tests', 'ldap_groups_test_db']),
  url = github_url,
  download_url = '%s/tarball/%s' % (github_url, version),
  keywords = ['django-auth-ldap', 'django', 'ldap'],
  license = 'LICENSE.txt',
  description = 'Django LDAP Group Mapper',
  long_description = long_description,
  zip_safe = False,
  install_requires = [
    'django-auth-ldap >= 1.2.0',
    'python-ldap >= 2.0',
    'Django >= 1.3']
)



from django.contrib import admin
from models import LDAPGroupMap

class LDAPGroupMapAdmin(admin.ModelAdmin):
    filter_horizontal = ('django_group',)
    search_fields = ['ldap_group',
                    'django_group__name']

admin.site.register(LDAPGroupMap, LDAPGroupMapAdmin)